// edtFTPnet
// 
// Copyright (C) 2004 Enterprise Distributed Technologies Ltd
// 
// www.enterprisedt.com
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// 
// Bug fixes, suggestions and comments should posted on 
// http://www.enterprisedt.com/forums/index.php
// 
// Change Log:
// 
// $Log: FTPFileFactory.cs,v $
// Revision 1.15  2007-05-23 00:28:32  hans
// Added TimeDifference and TimeIncludesSeconds properties.
//
// Revision 1.14  2007-03-16 04:57:49  bruceb
// VMSParser property
//
// Revision 1.13  2007/02/02 01:44:03  bruceb
// OS400 changes + use of RestartParsingException
//
// Revision 1.12  2006/11/17 15:38:42  bruceb
// rename Logger to string
//
// Revision 1.11  2006/07/12 08:24:46  bruceb
// added debug
//
// Revision 1.10  2006/06/14 10:08:21  bruceb
// VMS fixes
//
// Revision 1.9  2005/08/04 21:57:35  bruceb
// throw change
//
// Revision 1.8  2005/06/10 15:47:51  bruceb
// more vms tweaks
//
// Revision 1.7  2005/06/03 11:32:47  bruceb
// vms changes
//
// Revision 1.6  2005/02/07 17:19:34  bruceb
// support for setting the CultureInfo
//
// Revision 1.5  2004/12/22 22:41:50  bruceb
// made methods public
//
// Revision 1.4  2004/11/05 20:00:28  bruceb
// cleaned up namespaces
//
// Revision 1.3  2004/10/29 09:41:44  bruceb
// removed /// in file header
//
//
//

using System;
using System.Globalization;
using System.Text;
using System.ComponentModel;

using Logger = EnterpriseDT.Util.Debug.Logger;
	
namespace EnterpriseDT.Net.Ftp
{
	/// <summary>  
	/// Factory for creating FTPFile objects
	/// </summary>
	/// <author>       Bruce Blackshaw
	/// </author>
	/// <version>      $Revision: 1.15 $
	/// </version>
	public class FTPFileFactory
	{
        /// <summary>
		/// Get/set the culture info for parsing
		/// </summary>
		public CultureInfo ParsingCulture
		{
			get
			{
				return parserCulture;
			}
			set
			{
				this.parserCulture = value;
                windows.ParsingCulture = value;
                unix.ParsingCulture = value;
                vms.ParsingCulture = value;
                os400.ParsingCulture = value;
                if (parser != null)
                    parser.ParsingCulture = value;
			}			
		}        
        
		/// <summary> Logging object</summary>
		private Logger log = Logger.GetLogger("FTPFileFactory");
		
		/// <summary> Windows server comparison string</summary>
		internal const string WINDOWS_STR = "WINDOWS";
		
		/// <summary> UNIX server comparison string</summary>
		internal const string UNIX_STR = "UNIX";
        
		/// <summary> VMS server comparison string</summary>
		internal const string VMS_STR = "VMS";

        /// <summary> OS/400 server comparison string</summary>
        internal const string OS400_STR = "OS/400";
		
		/// <summary> SYST string</summary>
		private string system;
		
		/// <summary> Cached windows parser</summary>
		private WindowsFileParser windows = new WindowsFileParser();
		
		/// <summary> Cached unix parser</summary>
		private UnixFileParser unix = new UnixFileParser();

		/// <summary> Cached unix parser</summary>
		private VMSFileParser vms = new VMSFileParser();

        /// <summary> Cached unix parser</summary>
        private OS400FileParser os400 = new OS400FileParser();
		
		/// <summary> Does the parsing work</summary>
		private FTPFileParser parser = null;
		
		/// <summary> Rotate parsers when a ParseException is thrown?</summary>
		private bool rotateParsersOnFail = true;
    
        /// <summary>True if using VMS parser</summary>
        private bool usingVMS = false;
        
        /// <summary>Culture used for parsing file details</summary>
        private CultureInfo parserCulture = CultureInfo.InvariantCulture;

        /// <summary>Time difference between server and client (relative to client).</summary>
        protected TimeSpan timeDiff = new TimeSpan();

		/// <summary> Constructor
		/// 
		/// </summary>
		/// <param name="system">   SYST string
		/// </param>
		public FTPFileFactory(string system)
		{
			SetParser(system);
		}
		
		/// <summary> Constructor. User supplied parser. Note that parser
		/// rotation (in case of a ParseException) is disabled if
		/// a parser is explicitly supplied
		/// 
		/// </summary>
		/// <param name="parser">  the parser to use
		/// </param>
		public FTPFileFactory(FTPFileParser parser)
		{
			this.parser = parser;
			rotateParsersOnFail = false;
		}

        /// <summary> Default constructor. No parsers are set.
        /// </summary>
        public FTPFileFactory()
        {
        }

        /// <summary>
        /// Get or set the file parser to be used. If it is
        /// set explicitly, it is never rotated.
        /// </summary>
        [DefaultValue(null)]
        public FTPFileParser FileParser
        {
            get
            {
                return this.parser;
            }
            set
            {
                this.parser = value;
                if (value != null)
                    rotateParsersOnFail = false;
            }
        }

        /// <summary>
        /// Get the instance of the VMS parser
        /// </summary>
        public VMSFileParser VMSParser 
        {
            get 
            {
                return vms;
            }
        }

        /// <summary>
        /// [FTP/FTPS Only] Time difference between server and client (relative to client).
        /// </summary>
        /// <remarks>
        /// The time-difference is relative to the server such that, for example, if the server is
        /// in New York and the client is in London then the difference would be -5 hours 
        /// (ignoring daylight savings differences).  This property only applies to FTP and FTPS.
        /// </remarks>
        public TimeSpan TimeDifference
        {
            get { return timeDiff; }
            set { timeDiff = value; }
        }

        /// <summary>
        /// Get flag indicating whether or not the most recent parse returned seconds.
        /// </summary>
        public bool TimeIncludesSeconds
        {
            get { return parser.TimeIncludesSeconds; }
        }

		
		/// <summary> 
		/// Set the remote server type
		/// </summary>
		/// <param name="system">SYST string</param>
		public void SetParser(string system)
		{
			this.system = system.Trim();
            if (system.ToUpper().StartsWith(WINDOWS_STR))
            {
                log.Debug("Selected Windows parser");
                parser = windows;
            }
            else if (system.ToUpper().IndexOf(UNIX_STR) >= 0)
            {
                log.Debug("Selected UNIX parser");
                parser = unix;
            }
            else if (system.ToUpper().IndexOf(VMS_STR) >= 0) 
            {
                log.Debug("Selected VMS parser");
                parser = vms;
                usingVMS = true;
            }
            else if (system.ToUpper().IndexOf(OS400_STR) >= 0)
            {
                log.Debug("Selected OS/400 parser");
                parser = os400;
            }
            else 
            {
                parser = unix;
                log.Warn("Unknown SYST '" + system + "' - defaulting to Unix parsing");
            }
		}
		
		
		/// <summary> Parse an array of raw file information returned from the
		/// FTP server
		/// 
		/// </summary>
		/// <param name="files">    array of strings
		/// </param>
		/// <returns> array of FTPFile objects
		/// </returns>
		public virtual FTPFile[] Parse(string[] fileStrings)
		{			
            log.Debug("Parse() called using culture: " + parserCulture.EnglishName);
			FTPFile[] files = new FTPFile[fileStrings.Length];
			
			// quick check if no files returned
			if (fileStrings.Length == 0)
				return files;
			
			int count = 0;
            bool checkedUnix = false;
			for (int i = 0; i < fileStrings.Length; i++)
			{
                if (fileStrings[i] == null || fileStrings[i].Trim().Length == 0)
                    continue;
                
                // swap to Unix if looks like Unix listing - the default
                if (!checkedUnix && parser != unix && UnixFileParser.IsUnix(fileStrings[i])) {
                    parser = unix;
                    checkedUnix = true;
                    log.Info("Swapping parser to Unix");
                }                
                
                try
                {               
                    FTPFile file = null;
                    if (usingVMS) 
                    {
                        // vms uses more than 1 line for some file listings. We must keep going
                        // thru till we've got everything
                        StringBuilder filename = new StringBuilder(fileStrings[i]);
                        while (i+1 < fileStrings.Length && fileStrings[i+1].IndexOf(';') < 0) 
                        {
                            filename.Append(" ").Append(fileStrings[i+1]);
                            i++;
                        }
                        file = parser.Parse(filename.ToString());
                    }
                    else 
                    {
                        file = parser.Parse(fileStrings[i]);
                    }                
				
                    // we skip null returns - these are duff lines we know about and don't
                    // really want to throw an exception
                    if (file != null)
                    {
                        if (timeDiff.Ticks != 0)
                            file.ApplyTimeDifference(timeDiff);
                        files[count++] = file;
                    }
                }
                catch (RestartParsingException ex)
                {
                    log.Debug("Restarting parsing from first entry in list");
                    i = -1;
                    count = 0;
                    continue;
                }
                catch (FormatException ex)
                {
                    log.Debug(ex.Message);
                    if (rotateParsersOnFail && RotateParsers())
                    {		
                        parser.Parse(fileStrings[i]);
                        // succeeded, start again
                        i = -1; 
                        count = 0;
                        log.Debug("Restarting parsing from first entry in list");
                        continue; 
                    }
                    else // not rotating so rethrow (and fail)
                        throw;
                }
			}
			FTPFile[] result = new FTPFile[count];
			Array.Copy(files, 0, result, 0, count);
			return result;
		}
		
		/// <summary> 
		/// Swap from UNIX to Windows and vice versa. We don't bother
		/// rotating to the more obscure ones.
		/// </summary>
		private bool RotateParsers()
		{
			if (parser == unix)
			{
				parser = windows;
				log.Info("Rotated parser to Windows");
                return true;
			}
			else if (parser == windows)
			{
				parser = unix;
				log.Info("Rotated parser to UNIX");
                return true;
			}
            return false;
		}
		
		/// <summary> 
		/// Get the SYST string
		/// </summary>
		/// <returns> the system string.
		/// </returns>
        [Obsolete("Use the System property.")]
		public string GetSystem()
		{
			return system;
		}

        /// <summary>
        /// Get or set the system string (typically the string
        /// returned from the SYST command).
        /// </summary>
        [DefaultValue(null)]
        public string System
        {
            get
            {
                return system;
            }
            set 
            {
                if (value != null)
                    SetParser(value);
                else
                {
                    system = null;
                    parser = null;
                    usingVMS = false;
                    rotateParsersOnFail = true;                       
                }
            }
        }

	}
}
